@@ -7,8 +7,8 @@ import random
 
 import tkinter as tk
 from . import MODULES_VERBOSE
-from . import RootWindow, FrameUnlabelled, FrameLabelled, FrameStateful, FramePaned, Notebook, NotebookUniform
-from . import Button, Checkbox, Entry, Label, LabelStateful, Listbox, Combobox, ComboboxMap, EntryMultiline, Tree
+from . import RootWindow, FrameUnlabelled, FrameLabelled, FrameStateful, FramePaned, Notebook, NotebookUniform, ScrolledWidget
+from . import Button, Checkbox, Entry, Label, LabelStateful, Listbox, Combobox, ComboboxMap, EntryMultiline, Tree, ListboxControl
 from . import var, spec, fn
 from .model import CP, BindingGlobal, Timeout, TimeoutIdle, FileType, FileTypes
 
@@ -35,6 +35,8 @@ class LuftBaloons(FrameUnlabelled):
 
 
 class ListFrame_Inner(FrameStateful):
+    wstate_single = 'e'
+
     def __init__(self, *args, label, **kwargs):
         super().__init__(*args, label=label, labelInner=label, **kwargs)
 
@@ -57,7 +59,7 @@ class ListFrame_Outer(FrameStateful):
 
 
 class ListFrame_Lists(FramePaned):
-    layout = tk.VERTICAL
+    layout = tk.HORIZONTAL
 
     def setup_widgets(self, *, vLst):
         self.lstS = Listbox(self,
@@ -73,12 +75,13 @@ class ListFrame_Lists(FramePaned):
 
 
 class ListFrame(FrameLabelled):
-    layout = 'R2,1,2,1'
+    layout = 'R3,1,2,1'
 
     def setup_widgets(self, *, cbox1):
         i_choices: spec.StaticMap[int] = spec.StaticMapValues(lambda i: 'I%d' % i, range(10), defaultValue=7)
 
         self.bFill = Button(self, label='Fill Me!')
+        self.lPanes = Label(self, label='↓ Drag Separator ↓')
         self.bCheck = Button(self, label='Check')
 
         vLst = self.var(var.StringList, name='lst')
@@ -91,7 +94,7 @@ class ListFrame(FrameLabelled):
                                          cbox1=cbox1)
 
     def setup_layout(self, layout):
-        self.rowconfigure(self.cLst.grid_info().get('row'), weight=0)
+        fn.configure(self, self.cLst, rows=True, weight=0)
 
     def setup_defaults(self):
         self.fill_lst()
@@ -140,6 +143,7 @@ class UpstreamBool(FrameLabelled):
 
 class NB_Child_Simple(FrameUnlabelled):
     layout = 'Rx,1'
+    wstate_single = 'e'
 
     def setup_widgets(self, label):
         w = {}
@@ -329,6 +333,7 @@ class NB_Child_Timeouts(FrameUnlabelled):
 
 class NB_Child_Dialog(FrameUnlabelled):
     layout = 'Rx,1'
+    wstate_single = 'txt'
 
     def setup_widgets(self):
         self.ds = Button(self, label='D S')
@@ -369,31 +374,94 @@ class NB_Child_Dialog(FrameUnlabelled):
                         }))
 
 
+class NB_Child_Scrollbars(FrameUnlabelled):
+    layout = 'R2,x'
+    wstate_single = 'slst'
+
+    def setup_widgets(self):
+        self.randomize = Button(self, label='Randomize List Size')
+        self.setscrolls = Button(self, label='Toggle Scrollbars')
+
+        vSlist = self.var(var.StringList, name='slst')
+        self.sbL = ScrolledWidget(self, Listbox,
+                                  scrollHorizontal=None, scrollVertical=None,  # Auto (default)
+                                  height=5, variable=vSlist)
+        self.sbC = ScrolledWidget(self, Listbox,
+                                  scrollHorizontal=False, scrollVertical=False,  # Manual, disabled
+                                  height=5, variable=vSlist)
+        self.sbR = ScrolledWidget(self, Listbox,
+                                  scrollHorizontal=True, scrollVertical=True,  # Manual, enabled
+                                  height=5, variable=vSlist)
+
+        self.randomize.onClick = self.onRandom
+        self.setscrolls.onClick = self.onShowAll
+
+    def setup_adefaults(self):
+        self.onRandom()
+
+    def onRandom(self, event=None):
+        randomsize = random.randint(5, 30)  # Allow an opportunity for no vertical scrollbar
+        self.gvar('slst').set([f'Index {i:03}' for i in range(1, randomsize + 1)])
+
+    def onShowAll(self, event=None):
+        # sbL: Auto, Set True
+        self.sbL.set_scroll_state(True, True)
+        # sbC: Manual, Set True (no-op)
+        self.sbC.set_scroll_state(True, True)
+        # sbR: Manual, Set Reversed
+        self.sbR.set_scroll_state(*(not b for b in self.sbR.get_scroll_state()))
+
+
+class NB_Child_ListboxSet(FrameUnlabelled):
+    layout = tk.HORIZONTAL
+
+    def setup_widgets(self):
+        ls_all = spec.StaticList((f'String {idx:02}' for idx in range(1, 20 + 1)), defaultIndex=0)
+
+        ls_selected = random.sample(list(ls_all.all()), k=5)
+        self.left = ListboxControl(self,
+                                   selAll=ls_all)
+        self.right = ListboxControl(self, layout='c1,x', label='Full Control',
+                                    selAll=ls_all, selList=ls_selected,
+                                    buttonOne=False, buttonAll=True, buttonOrder=True)
+
+    def setup_defaults(self):
+        self.left.onAddAll()
+
+
 class NB(Notebook):
     def setup_tabs(self):
         logger.debug('» %r', self)
         # TODO: Test images, etc.
+
         return {
-            'tt': Notebook.Tab('Timeouts', NB_Child_Timeouts(self)),
-            'td': Notebook.Tab('Dialogues', NB_Child_Dialog(self)),
-            'tc': Notebook.Tab('Tab Complex', NB_Child_Complex(self)),
+            'sb': Notebook.Tab('Scrollbars', NB_Child_Scrollbars(self)),
+            'tt': Notebook.Tab('Timeouts', NB_Child_Timeouts(self),
+                               image=self.wimage('info-s16'), labelPosition=CP.E),  # Default labelPosition
+            'td': Notebook.Tab('Dialogues', NB_Child_Dialog(self),
+                               image=self.wimage('info-msgbox-s16'), labelPosition=True),  # Only image
+            'tc': Notebook.Tab('Tab Complex', NB_Child_Complex(self),
+                               image=self.wimage('info-s16'), labelPosition=CP.W),  # "Reverse" labelPosition
             't1': Notebook.Tab('Tab 1', NB_Child_Simple(self, label='Tab 1')),
             't2': Notebook.Tab('Tab 2', NB_Child_Simple(self, label='Tab 2')),
+            'c:lc': Notebook.Tab('Complex ListboxControl', NB_Child_ListboxSet(self)),
         }
 
     def setup_adefaults(self):
-        self.wselect('td')
+        self.wselect('c:lc')
 
 
 class TextEditor(FrameStateful):
     label = 'LTML'
     layout = 'Rx,1'
+    wstate_single = 'txt'
 
     def setup_widgets(self):
         self.bTxtClean = Button(self, label='Clean TXT')
         self.bTxtReset = Button(self, label='Reset TXT')
         self.bTxtSet = Button(self, label='Set TXT')
-        self.txt = EntryMultiline(self, setgrid=False)
+        self.txt = ScrolledWidget(self, EntryMultiline,
+                                  setgrid=False)
         # Setup events
         self.bTxtClean.onClick = self.txt.style_reset
         self.bTxtReset.onClick = self.txt_reset
@@ -433,7 +501,8 @@ class TextEditor(FrameStateful):
 
 
 class TreeExample(Tree):
-    pass
+    def onClickHeader(self, header):
+        logger.debug('Clicked on header "%s"', header.name)
 
 
 class ComboboxCB(Combobox):
@@ -451,7 +520,7 @@ class RW(RootWindow):
         self.b2 = Button(self, label='Debug')
         self.c2ro = Checkbox(self, label='RO "bool"', readonly=True, variable=vc)
         self.c2rw = Checkbox(self, label='RW "bool"', readonly=False, variable=vc)
-        self.e1 = Entry(self)  # label='Entry1'
+        self.e1 = Entry(self, expand=True)  # label='Entry1'
         self.lbs = LuftBaloons(self)
         self.ubox = UpstreamBool(self, label='Upstream', labelAnchor=CP.S,
                                  what_bool=vc)
@@ -461,9 +530,9 @@ class RW(RootWindow):
 
         self.txt = TextEditor(self)
         self.nb = NB(self)
-        self.arbre = TreeExample(self, label='Label', columns={
-            'number': 'Number',
-        })
+        self.arbre = TreeExample(self, label=Tree.Column('Label', image=self.wimage('warning-s16')), columns={
+            'number': Tree.Column('Number', image=self.wimage('info-s16'), nameAnchor=CP.W, cellAnchor=CP.E),
+        }, columns_stretch=None)
 
         # Setup events
         self.b1.onClick = self.c1.toggle
@@ -478,20 +547,20 @@ class RW(RootWindow):
     def setup_adefaults(self):
         logger.debug('Setup arbre state:')
         self.arbre.wstate = [
-            Tree.Element('First', ['1']),
-            Tree.Element('Second', ['2'], children=[
-                Tree.Element('Second.One', ['21']),
+            Tree.Element('First', ['1'], image=self.wimage('warning-s16')),
+            Tree.Element('Second', ['2'], image=self.wimage('warning-s16'), children=[
+                Tree.Element('Second.One', ['21'], image=self.wimage('error-s16')),
             ]),
-            Tree.Element('Third', ['3'], children=[
+            Tree.Element('Third', ['3'], image=self.wimage('warning-s16'), children=[
                 Tree.Element('Third.One', ['31']),
-                Tree.Element('Third.Two', ['32'], children=[
+                Tree.Element('Third.Two', ['32'], image=self.wimage('error-s16'), children=[
                     Tree.Element('Third.Two.One', ['321']),
-                    Tree.Element('Third.Two.Two', ['322']),
-                    Tree.Element('Third.Two.3', ['323']),
+                    Tree.Element('Third.Two.Two', ['322'], image=self.wimage('warning-s16')),
+                    Tree.Element('Third.Two.3', ['323'], image=self.wimage('error-s16')),
                 ]),
                 Tree.Element('Third.Tee', ['33']),
             ]),
-            Tree.Element('Tenth', ['10']),
+            Tree.Element('Tenth', ['10'], image=self.wimage('warning-s16')),
         ]
         logger.debug('Global Bindings:')
         for bname, B in self._bindings_global.items():
@@ -505,7 +574,7 @@ class RW(RootWindow):
     def debug(self):
         from pprint import pformat
         logging.info('=> State @ %s[%r]', self, self)
-        for line in pformat(self.wstate).splitlines():
+        for line in pformat(self.wstate_get()).splitlines():  # No `.wstate` for better tracebacks
             logging.info('%s', line)
         # logging.info('=> State @ ubox')
         # for line in pformat(self.ubox.wstate).splitlines():
@@ -513,6 +582,9 @@ class RW(RootWindow):
         # logging.info('=> State @ lf.cLst')
         # for line in pformat(self.lf.cLst.wstate).splitlines():
         #     logging.info('%s', line)
+        # logging.info('=> State @ nb[c:lc]')
+        # for line in pformat(self.nb.wtab('c:lc').wstate).splitlines():
+        #     logging.info('%s', line)
         logging.info('=> State Set')
         new = self.wstate
         for b in ('0', '12'):
@@ -547,12 +619,20 @@ def entrypoint():
     import argparse
     parser = argparse.ArgumentParser(description='Showcase for tkmilan module')
 
+    # ./showcase-images
+    default_images = Path(__file__).parent / 'showcase-images'
+
     # parser.add_argument('--version', action='version', version=PROJECT_VERSION)
     parser.add_argument('-v', '--verbose', dest='loglevel',
                         action='store_const', const=logging.DEBUG, default=logging.INFO,
                         help='Add more details to the standard error log')
     parser.add_argument('--debug', action='store_true',
                         help=argparse.SUPPRESS)
+    parser.add_argument('--images', type=Path, default=default_images,
+                        help='Image Folder to Load. Defaults to %(default)s')
+    parser.add_argument('--no-images', action='store_const',
+                        dest='images', const=None,
+                        help='Do not load any images')
     args = parser.parse_args()
 
     # Logs
@@ -569,7 +649,7 @@ def entrypoint():
             logging.getLogger(f'{__package__}.{dmodule}').setLevel(logging.INFO)
 
     # Widget Tester / Showcase
-    r = RW()
+    r = RW(imgfolder=args.images)
     logger.debug('Screen Size: %r', r.size_screen)
     logger.debug('         /2: %r', r.size_screen.reduce(2))
     r.mainloop()
