@@ -47,28 +47,28 @@ def grid_size(*widgets: 'mixin.MixinWidget') -> 'model.GridSize':
     Args:
         widgets: Widgets in the same grid. There should be at least one.
     """
-    def maxs(w):
-        assert isinstance(w, tk.Widget), f'Invalid Widget: {w!r}'
-        info = w.grid_info()
+    def maxs(w: 'mixin.MixinWidget') -> typing.Tuple[int, int]:
+        info = w.wgrid
         # logger.debug(f'=> Grid Info: {info}')
-        # If the grid information doesn't exist, default to a single frame
-        # Force elements to integer, on tcl v8.5 they are returned as strings
-        r = int(info.get('row', 0)) + int(info.get('rowspan', 1)) - 1
-        c = int(info.get('column', 0)) + int(info.get('columnspan', 1)) - 1
-        return (r, c)
+        if info is None:
+            return (-1, -1)  # Not included on a grid yet
+        else:
+            return (info.row + info.rowspan - 1, info.column + info.columnspan - 1)
     if __debug__:
         parents = set()
         for w in widgets:
             if w.wparent:
                 parents.add(w.wparent)
-        assert len(parents) == 1, f'Grid Size only for sibling widgets. Parents {parents}'
+        assert len(parents) == 1, f'Grid Size only for sibling widgets. Parents: {parents}'
     m = [maxs(w) for w in widgets]
     num_columns = max([w[1] for w in m]) + 1
     num_rows = max([w[0] for w in m]) + 1
     return model.GridSize(rows=num_rows, columns=num_columns)
 
 
-def configure_grid(master: 'mixin.ContainerWidget', column_weights: typing.Sequence[int], row_weights: typing.Sequence[int], **kwargs) -> None:
+def configure_grid(master: 'mixin.ContainerWidget',
+                   column_weights: typing.Sequence[int], row_weights: typing.Sequence[int],
+                   **kwargs: typing.Mapping[str, typing.Any]) -> None:
     """Configure the grid.
 
     Weights can be:
@@ -76,14 +76,15 @@ def configure_grid(master: 'mixin.ContainerWidget', column_weights: typing.Seque
         - ``0`` : Fit the widgets, never resize
         - ``>0``: Resize with this number as weight
 
-    Make sure to include all columns and rows. When in doubt, use 0
+    Make sure to include all columns and rows. When in doubt, use 0.
 
     Args:
         column_weights: List of column weights
         row_weights: List of row weights
-        kwargs: Extra arguments to `columnconfigure
-            <https://www.tcl.tk/man/tcl/TkCmd/grid.html#M8>`_/`rowconfigure
-            <https://www.tcl.tk/man/tcl/TkCmd/grid.html#M24>`_ function.
+        kwargs: Extra arguments to the configuration functions
+            `columnconfigure <https://www.tcl.tk/man/tcl/TkCmd/grid.html#M8>`_
+            /
+            `rowconfigure <https://www.tcl.tk/man/tcl/TkCmd/grid.html#M24>`_.
     """
     if __debug__:
         gw = master.gsize
@@ -91,9 +92,38 @@ def configure_grid(master: 'mixin.ContainerWidget', column_weights: typing.Seque
         assert gw == gr, f'{master!r}: Invalid grid size: W::{gw} R::{gr}'
     assert isinstance(master, (tk.Widget, tk.Tk)), f'{master} is not a valid tkinter.Widget'
     for col, w in enumerate(column_weights):
-        master.columnconfigure(col, weight=w, **kwargs)
+        master.columnconfigure(col, weight=w, **kwargs)  # type: ignore  # Invalid Types
     for row, h in enumerate(row_weights):
-        master.rowconfigure(row, weight=h, **kwargs)
+        master.rowconfigure(row, weight=h, **kwargs)  # type: ignore  # Invalid Types
+
+
+def configure(master: 'mixin.ContainerWidget',
+              *widgets: 'mixin.MixinWidget',
+              c_rows: bool = False, c_cols: bool = False,
+              **kwargs: typing.Mapping[str, typing.Any]) -> None:
+    '''Configure the rows and columns for the given widgets.
+
+    For widgets that span more than one row or column, the settings are changed
+    for all rows or columns.
+
+    Args:
+        widgets: Widgets to consider.
+        c_rows: Configure the rows
+        c_cols: Configure the columns
+        kwargs: Arguments passed to the configuration functions:
+            `columnconfigure <https://www.tcl.tk/man/tcl/TkCmd/grid.html#M8>`_
+            /
+            `rowconfigure <https://www.tcl.tk/man/tcl/TkCmd/grid.html#M24>`_.
+    '''
+    assert isinstance(master, (tk.Widget, tk.Tk)), f'{master} is not a valid tkinter.Widget'
+    for w in widgets:
+        wgrid = w.wgrid
+        if wgrid and c_rows:
+            for r in range(wgrid.row, wgrid.row + wgrid.rowspan):
+                master.rowconfigure(r, **kwargs)  # type: ignore  # Invalid Types
+        if wgrid and c_cols:
+            for c in range(wgrid.column, wgrid.column + wgrid.columnspan):
+                master.columnconfigure(c, **kwargs)  # type: ignore  # Invalid Types
 
 
 def generate_trace(variable: tk.Variable, function: typing.Callable, **kwargs):
