@@ -4,44 +4,25 @@ All the mixin classes, to be reused internally
 import logging
 import warnings
 import typing
-import math
 from dataclasses import dataclass
 import collections
 from functools import cached_property, partial
 
-from . import exception
 from . import fn
 from . import model
 from . import var
+from . import autolayout
 
 import tkinter as tk
-from tkinter import HORIZONTAL, VERTICAL
 
 if typing.TYPE_CHECKING:
-    from . import RootWindow
+    from . import RootWindow, ScrolledWidget
 
 logger = logging.getLogger(__name__)
-logger_layout = logging.getLogger('%s.layout' % __name__)
 
-# Automatic Layout
-# - HORIZONTAL
-# - VERTICAL
-AUTO: str = 'auto'
-LAYOUT_SYNONYMS: typing.Mapping[str, str] = {
-    HORIZONTAL: '1x',  # 1 Row
-    VERTICAL: 'x1',  # 1 Column
-    AUTO: 'x',  # Square
-}
-# Multiples
-# - Use the direction names directly? With a prefix?
-LAYOUT_MULTIPLE: typing.Mapping[str, model.Direction] = {
-    'R': model.Direction.E,
-    'r': model.Direction.W,
-    'C': model.Direction.S,
-    'c': model.Direction.N,
-}
 WEIRD_WIDGET_NAME = [  # Weird Widget `dir` names, these cause trouble
     '_last_child_ids',
+    'wroot',
 ]
 
 
@@ -169,6 +150,17 @@ class MixinStateContainer(MixinState):
         dictionary mapping subwidget identifier to `WidgetDynamicState`.
     '''
     wstate_static: bool = False
+    wstate_single: typing.Optional[str] = None
+    '''
+    Mark the container state as "single", including only the state for this child.
+
+    Should only be enabled where there is a single child element, this is
+    verified when getting the value. Use `MixinWidget.ignoreContainerState` to
+    ignore other widgets.
+
+    This creates no performance improvements, it is only useful to simplify the
+    state.
+    '''
 
     def state_get(self, **kwargs):
         state = {}
@@ -178,34 +170,47 @@ class MixinStateContainer(MixinState):
                 pass  # Skip
             else:
                 state[identifier] = result
-        if len(state) == 0:
-            return None
+        if self.wstate_single is not None:
+            assert len(state) == 1, f'{self}: Invalid "wstate_single" marking, {len(state)} elements'
+            return state[self.wstate_single]
         else:
-            return state
+            if len(state) == 0:
+                return None
+            else:
+                if __debug__:
+                    if len(state) == 1:
+                        logger.warning('%s: This widget can be marked "wstate_single": `%s`', self, list(state)[0])
+                return state
 
     def state_set(self, state, substate, **kwargs) -> None:
         # # Debug container state flow
         # self_names = None
         # if __debug__:
         #     self_names = str(self).split('.')[1:]
-        #     logger.debug('%s: %s', '>' * len(self_names), self)
+        #     logger.debug('%s: %s%s',
+        #                  '>' * len(self_names),
+        #                  self,
+        #                  '' if self.wstate_single is None else f' [{self.wstate_single}]',
+        #                  )
         for identifier, wds in self.stateSetup(**kwargs).items():
-            if (wds.noneable and state is None) or ((substate or wds.noneable) and identifier not in state):
+            skip = (wds.noneable and state is None) or \
+                ((substate or wds.noneable) and identifier not in state) or \
+                (self.wstate_single is not None and self.wstate_single != identifier)
+            if skip:
                 # if __debug__:
                 #     logger.debug('%s|> Skip "%s"', ' ' * len(self_names), identifier)
                 pass
             else:
-                if wds.container:
-                    wds.setter(state[identifier], substate, **kwargs)
+                # if __debug__:
+                #     logger.debug('%s|>  Set "%s"', ' ' * len(self_names), identifier)
+                if self.wstate_single:
+                    widget_state = state
                 else:
-                    wds.setter(state[identifier])
-
-    def IgnoreWidgets(self, *ignored):
-        '''
-        Wrap `MixinState.setup_state`, ignoring some widgets.
-        '''
-        state = super().setup_state()
-        return {i: w for i, w in state if w not in ignored}
+                    widget_state = state[identifier]
+                if wds.container:
+                    wds.setter(widget_state, substate, **kwargs)
+                else:
+                    wds.setter(widget_state)
 
 
 class MixinWidget:
@@ -225,6 +230,8 @@ class MixinWidget:
     '''The supported `GuiState` keys.'''
     isHelper: bool
     '''Marker that indicates the widgets is not part of the automatic state.'''
+    ignoreContainerState: bool = False
+    '''Ignore this widget's state when included on a container.'''
     _bindings: typing.MutableMapping[str, model.Binding]
     '''Store all widget `Binding` objects, keyed by name (see `binding`).'''
     _tidles: typing.MutableMapping[str, model.TimeoutIdle]
@@ -347,6 +354,9 @@ class MixinWidget:
         Args:
             fmt: The grid configuration format. Specified above.
             kwargs: Passed upstream
+
+        See Also:
+            `wgrid`: Get the current widget grid coordinates.
         '''
         assert isinstance(self, (tk.Widget, tk.Tk)), f'{self} is not a valid tkinter.Widget'
         if isinstance(fmt, str):
@@ -354,6 +364,39 @@ class MixinWidget:
         kwargs.update(fmt.dict())
         self.grid(**kwargs)
 
+    @property
+    def wgrid(self) -> typing.Optional[model.GridCoordinates]:
+        '''Get the widget grid coordinates, if the widget is visible.
+
+        Returns:
+            Return a `model.GridCoordinates` object with the widget information. If
+            the wiget was hidden, return `None`.
+
+            This is also available for the root widget (`wroot`) for
+            completeness, but that doesn't really correspond to any grid,
+            return `None`.
+
+        See Also:
+            `setup_grid`: Change the widget grid coordinates.
+        '''
+        if self == self.wroot:
+            assert isinstance(self, tk.Tk)
+            return None
+        else:
+            assert isinstance(self, tk.Widget), f'{self} is not a valid tkinter.Widget'
+            # If the grid information doesn't exist, default to a single frame
+            # Force elements to integer, on tcl v8.5 they are returned as strings
+            ginfo = self.grid_info()
+            if ginfo:
+                return model.GridCoordinates(
+                    row=int(ginfo.get('row', 0)),
+                    rowspan=int(ginfo.get('rowspan', 1)),
+                    column=int(ginfo.get('column', 0)),
+                    columnspan=int(ginfo.get('columnspan', 1)),
+                )
+            else:
+                return None
+
     def get_gui_state(self) -> model.GuiState:
         if __debug__:
             from . import RootWindow  # For typechecking
@@ -429,6 +472,8 @@ class SingleWidget(MixinWidget, MixinStateSingle, MixinTraces):
     '''Parent class of all single widgets.'''
     variable: typing.Optional[var.Variable] = None
     state_type: typing.Optional[typing.Type[var.Variable]] = None
+    scrollFrame: 'typing.Optional[ScrolledWidget]' = None
+    '''Link to the corresponding `ScrolledWidget`, if exists'''
 
     def init_single(self, variable: var.Variable = None, gkeys: typing.Iterable[str] = None) -> None:
         '''Setup all single widget stuff.'''
@@ -449,6 +494,10 @@ class SingleWidget(MixinWidget, MixinStateSingle, MixinTraces):
     def setup_state(self):
         return self.variable
 
+    def wimage(self, key: str) -> typing.Optional[tk.Image]:
+        '''Wraper for `RootWindow.wimage`.'''
+        return self.wroot.wimage(key)
+
     # TODO: Move here from Combobox
     # specValues: typing.Optional[spec.Spec]
     # def setDefault(self) -> None:
@@ -517,7 +566,7 @@ class ContainerWidget(MixinWidget, MixinStateContainer):
             chosen_layout = self.layout
         else:
             # Fallback
-            chosen_layout = AUTO
+            chosen_layout = autolayout.AUTO
         self.layout_container(chosen_layout, expand, autogrow)
         self.setup_defaults()
         self.after_idle(lambda: self.setup_adefaults())  # No need for a `TimeoutIdle` here
@@ -572,114 +621,16 @@ class ContainerWidget(MixinWidget, MixinStateContainer):
             assert isinstance(self, tk.ttk.Widget), f'{self} is not a valid tkinter.ttk.Widget'
             self.grid(sticky=tk.NSEW)
         # Automatic Layout
-        # TODO: Test and document this
+        layout, args = autolayout.do(layout, len(self.widgets))
         if layout:
-            layout_given = layout
-            layout = LAYOUT_SYNONYMS.get(layout, layout)
-            assert layout is not None, f'Invalid Layout: {layout}'
-            direction_names = tuple(d.name for d in model.Direction)
-            try:
-                amount = len(self.widgets)
-                args: typing.MutableSequence[model.GridCoordinates] = []
-                if layout.startswith(('x', 'X')):
-                    auto_separator = layout[0]
-                    if layout[1:] in ('', *direction_names):
-                        square = math.ceil(math.sqrt(amount))
-                        logger_layout.debug('Layout: Square (%d)', square)
-                        layout = layout.replace(auto_separator, '%d%s%d' % (square, auto_separator, square), 1)
-                        logger_layout.debug('      : %s', layout)
-                if layout.startswith(tuple(LAYOUT_MULTIPLE)):
-                    _type = layout[0]
-                    multiple_direction = LAYOUT_MULTIPLE[_type]
-                    logger_layout.debug('Layout: Multiples %s: %s', _type, multiple_direction)
-
-                    def parse_amount(value):
-                        # Allow 'x' to be replaced with the remaining widgets
-                        if value == 'x':
-                            return None
-                        else:
-                            int_value = int(value)
-                            if int_value == 0:
-                                raise ValueError('Layout Multiples: Invalid Value: %s' % value)
-                            return int_value
-                    amounts = [parse_amount(v) for v in layout[1:].split(',')]
-                    args.extend(multiple_direction.multiples(*amounts, amount=amount))
-                elif 'x' in layout or 'X' in layout:
-                    auto_direction = model.Direction.S  # Default automatic Direction
-                    if any(layout.endswith(name) for name in direction_names):
-                        # Requires the Direction name to be 1 character long
-                        auto_direction = model.Direction[layout[-1]]
-                        layout = layout[:-1]
-                    auto_force: bool = 'X' in layout
-                    assert isinstance(auto_direction, model.Direction)
-                    if auto_force:
-                        auto_separator = 'X'
-                        assert 'x' not in layout
-                    else:
-                        auto_separator = 'x'
-                        assert 'X' not in layout
-                    rows, cols = [None if v == '' else int(v) for v in layout.split(auto_separator)]
-                    # At least one of `rows`/`cols` is not `None`
-                    if rows is None:
-                        assert cols is not None
-                        rows = math.ceil(amount / cols)
-                    if cols is None:
-                        assert rows is not None
-                        cols = math.ceil(amount / rows)
-                    grid_missing = rows * cols - amount
-                    if grid_missing > 0 and not auto_force:
-                        if grid_missing >= cols:
-                            dcols = grid_missing // cols
-                            logger_layout.debug('      : -%d Columns', dcols)
-                            cols -= dcols
-                            grid_missing = rows * cols - amount
-                            if __debug__:
-                                if layout_given not in LAYOUT_SYNONYMS:
-                                    # This might be a spurious warning
-                                    warnings.warn('Non-automatic layout being unsquared: %d cols' % dcols, stacklevel=4)
-                        if grid_missing >= rows:
-                            drows = grid_missing // rows
-                            logger_layout.debug('      : -%d Rows', drows)
-                            rows -= drows
-                            grid_missing = rows * cols - amount
-                            if __debug__:
-                                if layout_given not in LAYOUT_SYNONYMS:
-                                    # This might be a spurious warning
-                                    warnings.warn('Non-automatic layout being unsquared: %d rows' % drows, stacklevel=4)
-                    logger_layout.debug('Layout: Automatic Grid (%d%s%d%s)[%+d]', rows, auto_separator, cols, auto_direction.name, grid_missing)
-                    args.extend(auto_direction.grid(rows, cols, amount=amount,
-                                                    auto_fill=not auto_force))
-                if __debug__:
-                    logger_layout.debug(f'{self}: => {amount} widgets')
-                container_matrix = None  # For debug
-                if __debug__:
-                    try:
-                        from defaultlist import defaultlist  # type: ignore
-                        container_matrix = defaultlist(lambda: defaultlist())
-                    except ImportError:
-                        pass  # Don't use if it doesn't exist
-                for idx, (arg, widget) in enumerate(zip(args, self.widgets.values())):
-                    assert isinstance(widget, tk.Widget)
-                    if __debug__:
-                        if container_matrix is not None:
-                            for drow in range(arg.rowspan):
-                                for dcol in range(arg.columnspan):
-                                    container_matrix[arg.row + drow][arg.column + dcol] = idx
-                        else:
-                            logger_layout.debug('| %s' % arg)
-                    widget.grid(**arg.dict())
-                if __debug__:
-                    if container_matrix is not None:
-                        for r in container_matrix:
-                            assert isinstance(r, typing.Sequence)
-                            logger_layout.debug('| %s', ' '.join(('x' * 2 if i is None else '%02d' % i for i in r)))
-            except Exception:
-                raise exception.InvalidLayoutError('Layout: %s' % layout)
+            # if __debug__:
+            #     logger.debug(f'{self}: => {len(self.widgets)} widgets')
+            for idx, (arg, widget) in enumerate(zip(args, self.widgets.values())):
+                assert isinstance(widget, tk.Widget)
+                widget.grid(**arg.dict())
         # TODO: Return a boolean/dataclass to control autogrowth
         self.setup_layout(layout)
         self.layout = layout  # Setup the final layout setting
-        if __debug__:
-            logger_layout.debug(f'Final Layout: {self.layout}')
         if autogrow:
             if size := self.gsize:
                 fn.configure_grid(self, [1] * size.columns, [1] * size.rows)
@@ -699,6 +650,9 @@ class ContainerWidget(MixinWidget, MixinStateContainer):
         # logger.debug('%r START | %r', self, vid_upstream)
         for name, widget in self.widgets.items():
             # logger.debug('%s: %r', name, widget)
+            if widget.ignoreContainerState:
+                # logger.debug('| Skipping Widget')
+                continue
             if isinstance(widget, SingleWidget):
                 assert widget.variable is not None
                 vid = fn.vname(widget.variable)
@@ -807,3 +761,7 @@ class ContainerWidget(MixinWidget, MixinStateContainer):
             all widgets are stable.
         '''
         pass
+
+    def wimage(self, key: str) -> typing.Optional[tk.Image]:
+        '''Wraper for `RootWindow.wimage`.'''
+        return self.wroot.wimage(key)
