@@ -32,6 +32,25 @@ There is no Python documentation, see ``Tk`` `trace variable <https://www.tcl.tk
 '''
 
 
+# Image Types
+__image_types = {
+}
+# TODO: Support more image types with Pillow:
+# - See https://pillow.readthedocs.io/en/stable/reference/ImageTk.html
+# PNG is only supported on "tk8.6"
+# - Gated by Tk version
+if util.TK_VERSION >= (8, 6):
+    __image_types['png'] = tk.PhotoImage
+# GIF is supported everywhere
+__image_types['gif'] = tk.PhotoImage
+
+IMAGE_TYPES = __image_types
+'''Supported image types, and corresponding loaders.
+
+See `ImageCache`.
+'''
+
+
 logger = logging.getLogger(__name__)
 logger_layout = logging.getLogger('%s.layout' % __name__)
 
@@ -76,7 +95,7 @@ class WStyle:
 
     Args:
         _default: Does this represent a default value?
-            When this is set (only for class ``__init___`` definitions), the
+            When this is set (only for class ``__init__`` definitions), the
             values can be safely overriden.
 
     Note:
@@ -96,13 +115,23 @@ class CP(enum.Enum):
     # value: typing.Optional[str]  # TODO: Improve labelAnchor usages?
 
     N = tk.N
+    '''North'''
     S = tk.S
+    '''South'''
     E = tk.E
+    '''East'''
     W = tk.W
+    '''West'''
     NE = tk.NE
+    '''NorthEast = `N` + `E`'''
     NW = tk.NW
+    '''NorthWest = `N` + `W`'''
     SE = tk.SE
+    '''SouthEast = `S` + `E`'''
     SW = tk.SW
+    '''SouthWest = `S` + `W`'''
+    center = tk.CENTER
+    '''Center element on the container. Not a "cardinal point" in the usual sense.'''
     default = None
     '''OS-specific cardinal point.'''
 
@@ -116,7 +145,7 @@ CP_Compound: typing.Mapping[typing.Union[CP, bool, None], str] = {
     # Special Compound values
     True: 'image',  # No Label, only Image
     False: 'text',  # No Image, only Label
-    None: 'center',  # Image and Label, centered
+    None: tk.CENTER,  # Image and Label, centered
 }
 '''The ``compound`` configuration setting.
 
@@ -211,20 +240,46 @@ class GridSize:
     '''Get this information as a tuple.'''
 
 
+@dataclass
+class ImageCache:
+    '''Image Cache metadata.
+
+    Each cached image has metadata associated with it, to be able to support
+    lazy-loading images.
+
+    The ``obj`` point to the image object, the rest are metadata used to
+    reconstruct it.
+    '''
+    # Cached
+    obj: typing.Optional[tk.Image] = None
+    # UnCached
+    fname: typing.Optional[Path] = None
+    data: typing.Optional[bytes] = None
+    dtype: typing.Optional[str] = None
+
+    def __post_init__(self):
+        assert self.dtype is None or self.dtype in IMAGE_TYPES
+
+    @property
+    def cached(self) -> bool:
+        '''Check if the image object is cached.'''
+        return self.obj is not None
+
+
 class Direction(enum.Enum):
     '''Hold the direction of automatic widget layout.
 
     The possible values are the cardinal directions:
 
-    +---+---+---+
-    |  Cardinal |
-    +---+---+---+
-    |   | N |   |
-    +---+---+---+
-    | W |   | E |
-    +---+---+---+
-    |   | S |   |
-    +---+---+---+
+    +-----+-----+-----+
+    |*Cardinal Point* |
+    +-----+-----+-----+
+    |     |  N  |     |
+    +-----+-----+-----+
+    |  W  |     |  E  |
+    +-----+-----+-----+
+    |     |  S  |     |
+    +-----+-----+-----+
 
     - **N**: North
     - **S**: South
@@ -1014,7 +1069,7 @@ class NotebookTab:
     '''
     name: str
     widget: 'mixin.ContainerWidget'
-    image: typing.Optional[tk.PhotoImage] = None
+    image: typing.Optional[tk.Image] = None
     extra: typing.Mapping[str, typing.Any] = dc_field(default_factory=dict)
     labelPosition: InitVar[CP] = CP.E  # init-only
     identifier: typing.Optional[str] = dc_field(default=None, init=False)
@@ -1031,16 +1086,33 @@ class NotebookTab:
 
 @dataclass
 class TreeColumn:
-    '''Information for each `tkmilan.Tree` column heading.
+    '''Information for each `tkmilan.Tree` column.
+
+    This includes settings for both the column heading, and the column cells.
 
     Args:
-        identifier: The id string. This is not shown to the user
         name: The heading name, shown on the top of the column
+        identifier: The id string. This is not shown to the user.
+            Technically optional, but only when defining the columns, it will
+            always have a `str` in normal circumstances.
+        stretch: Adjust the column size when resizing the widget. Defaults to
+            `True`, automatic resizes.
+        image: Show an icon on the right side of the heading. Optional.
+        nameAnchor: Anchor the heading name to a `CP`. Defaults to `CP.center`.
+        cellAnchor: Anchor the body column cells to a `CP`. Defaults to `CP.W`.
     '''
-    identifier: str
     name: str
-    # TODO: image
-    # TODO: anchor: CP
+    identifier: typing.Optional[str] = None
+    stretch: bool = True
+    image: typing.Optional[tk.Image] = None
+    nameAnchor: CP = CP.center
+    cellAnchor: CP = CP.W
+
+    def __post_init__(self):
+        if self.nameAnchor == self.nameAnchor.default:
+            raise ValueError(f'Invalid CP for "nameAnchor": {self.nameAnchor}')
+        if self.cellAnchor == self.cellAnchor.default:
+            raise ValueError(f'Invalid CP for "cellAnchor": {self.cellAnchor}')
 
 
 @dataclass
@@ -1050,6 +1122,9 @@ class TreeElement:
     Technically, this is not enough to reconstruct the entire tree, but it
     should.
 
+    The ``image`` argument is optional, but when created, no resizing is performed.
+    By default, the ideal image size is ``16x16`` pixels.
+
     Args:
         label: The leftmost string shown on the first column. This identifies the entire record.
         columns: A list of strings corresponding to the column data.
@@ -1057,12 +1132,16 @@ class TreeElement:
         children: A list of `TreeElement`, to be shown as children of this
             `TreeElement`. This can recurse without limit.
             Optional, defaults to `None`.
+        tags: Record tags. Optional, defaults to an empty list.
+        image: Show an icon on the left side of the record. Optional.
         data: Arbitrary data to store on the element.
             Optional, defaults to `None`.
     '''
     label: str
     columns: typing.Optional[typing.Sequence[str]] = None
     children: 'typing.Optional[typing.Sequence[TreeElement]]' = None
+    tags: typing.Optional[typing.Sequence[str]] = None
+    image: typing.Optional[tk.Image] = None
     data: typing.Any = None
 
 
